// Copyright (C) 2024 Simon Quigley <tsimonq2@ubuntu.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <https://www.gnu.org/licenses/>.

#ifndef AUTHENTICATION_H
#define AUTHENTICATION_H

#include <string>
#include <map>

#ifndef LAUNCHPAD_API
#ifdef BUILDING_LAUNCHPAD
  #define LAUNCHPAD_API __attribute__((visibility("default")))
#else
  #define LAUNCHPAD_API
#endif
#endif

extern const std::string AUTH_PLAINTEXT_CREDENTIALS_FILE;

bool read_plaintext_credentials_impl(std::string& consumer_key,
                                     std::string& consumer_secret,
                                     std::string& oauth_token,
                                     std::string& oauth_token_secret);

bool create_plaintext_credentials_impl(const std::string& consumer_key,
                                       const std::string& consumer_secret,
                                       const std::string& oauth_token,
                                       const std::string& oauth_token_secret);

bool read_gnome_keyring_impl(std::string& consumer_key,
                             std::string& consumer_secret,
                             std::string& oauth_token,
                             std::string& oauth_token_secret);

bool create_keyring_credentials_impl(const std::string& consumer_key,
                                     const std::string& consumer_secret,
                                     const std::string& oauth_token,
                                     const std::string& oauth_token_secret);

std::string build_oauth_header_impl(const std::string& consumer_key,
                                    const std::string& oauth_token,
                                    const std::string& oauth_token_secret,
                                    const std::string& http_method,
                                    const std::string& url,
                                    const std::map<std::string, std::string>& additional_params);

bool open_url_impl(const std::string& url);
std::map<std::string, std::string> parse_response(const std::string& response, std::map<std::string, std::string>& params);

#endif // AUTHENTICATION_H
