#ifndef BINARY_PACKAGE_PUBLISHING_HISTORY_H
#define BINARY_PACKAGE_PUBLISHING_HISTORY_H

#include <string>
#include <optional>
#include <nlohmann/json.hpp>
#include <generator>
#include "build.h"

#ifndef LAUNCHPAD_API
#ifdef BUILDING_LAUNCHPAD
  #define LAUNCHPAD_API __attribute__((visibility("default")))
#else
  #define LAUNCHPAD_API
#endif
#endif

class launchpad;

class binary_package_publishing_history {
public:
    binary_package_publishing_history();
    ~binary_package_publishing_history();

    static std::optional<binary_package_publishing_history> parse(const std::string& json_data);
    void parse_json(const std::string& json_data);
    void set_lp(launchpad* lp_ptr);

    bool architecture_specific;
    std::string archive_link;
    std::string binary_package_name;
    std::string binary_package_version;
    std::string build_link;
    std::string component_name;
    std::string copied_from_archive_link;
    std::string creator_link;
    std::string date_created;
    std::string date_made_pending;
    std::string date_published;
    std::string date_removed;
    std::string date_superseded;
    std::string display_name;
    std::string distro_arch_series_link;
    std::string http_etag;
    bool is_debug;
    int phased_update_percentage; // -1 if None
    std::string pocket;
    std::string priority_name;
    std::string removal_comment;
    std::string removed_by_link;
    std::string resource_type_link;
    std::string scheduled_deletion_date;
    std::string section_name;
    std::string self_link;
    std::string source_package_name;
    std::string source_package_version;
    std::string status;

    std::optional<binary_package_publishing_history> get() const;
    bool patch(const nlohmann::json& data) const;
    bool put(const nlohmann::json& data) const;

    std::optional<nlohmann::json> binaryFileUrls(bool include_meta=false) const;
    std::optional<nlohmann::json> getDailyDownloadTotals(const std::string& start_date="", const std::string& end_date="") const;
    std::optional<int> getDownloadCount() const;
    std::optional<nlohmann::json> getDownloadCounts(const std::string& start_date="", const std::string& end_date="") const;

    std::optional<binary_package_publishing_history> changeOverride(const std::optional<std::string>& new_component,
                                                                    const std::optional<int>& new_phased_update_percentage,
                                                                    const std::optional<std::string>& new_priority,
                                                                    const std::optional<std::string>& new_section) const;
    bool requestDeletion(const std::optional<std::string>& removal_comment) const;
    std::optional<class build> build = getBuild();

private:
    const std::optional<class build> getBuild();
    launchpad* lp;
};

#endif // BINARY_PACKAGE_PUBLISHING_HISTORY_H
