#ifndef DISTRO_ARCH_SERIES_H
#define DISTRO_ARCH_SERIES_H

#include <string>
#include <optional>
#include <nlohmann/json.hpp>
#include <generator>

#ifndef LAUNCHPAD_API
#ifdef BUILDING_LAUNCHPAD
  #define LAUNCHPAD_API __attribute__((visibility("default")))
#else
  #define LAUNCHPAD_API
#endif
#endif

class launchpad;

class distro_arch_series {
public:
    distro_arch_series();
    ~distro_arch_series();

    static std::optional<distro_arch_series> parse(const std::string& json_data);
    void parse_json(const std::string& json_data);
    void set_lp(launchpad* lp_ptr);

    // Writable fields
    std::string architecture_tag;
    std::string display_name;
    std::string distroseries_link;
    bool enabled;
    bool is_nominated_arch_indep;
    std::string main_archive_link;
    bool official;
    std::string owner_link;
    int package_count;
    std::string title;

    // Read-only fields
    std::string chroot_url;
    std::string http_etag;
    std::string processor_link;
    std::string resource_type_link;
    std::string self_link;
    bool supports_virtualized;
    std::string web_link;

    std::optional<distro_arch_series> get() const;
    bool patch(const nlohmann::json& data) const;
    bool put(const nlohmann::json& data) const;

    // Custom GET methods
    std::generator<nlohmann::json> getBuildRecords(const std::string& build_state="",
                                                   const std::string& pocket="",
                                                   const std::string& source_name="") const;
    std::optional<nlohmann::json> getChrootHash(const std::string& image_type="Chroot tarball",
                                                const std::string& pocket="Release") const;
    std::optional<std::string> getChrootURL(const std::optional<std::string>& image_type=std::nullopt,
                                            const std::optional<std::string>& pocket=std::nullopt) const;
    std::optional<nlohmann::json> getSourceFilter() const;

    // Custom POST methods
    bool removeChroot(const std::optional<std::string>& image_type=std::nullopt,
                      const std::optional<std::string>& pocket=std::nullopt) const;
    bool removeSourceFilter() const;
    bool setChroot(const std::string& data_link,
                   const std::optional<std::string>& image_type=std::nullopt,
                   const std::optional<std::string>& pocket=std::nullopt,
                   const std::string& sha1sum="") const;
    bool setChrootFromBuild(const std::string& filename,
                            const std::string& livefsbuild_link,
                            const std::optional<std::string>& image_type=std::nullopt,
                            const std::optional<std::string>& pocket=std::nullopt) const;
    bool setSourceFilter(const std::string& packageset,
                         const std::string& sense) const;

private:
    launchpad* lp;
};

#endif // DISTRO_ARCH_SERIES_H
